#include <glob.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <inttypes.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libcowonilb.h"

#define LICENCEPATH "./LICENCE"	// set this in ./configure

GtkBuilder *builder;			// main gui object
char* last_filename = NULL;		// name of the last opened ilb animation
CowonAnim *animation = NULL;	// the animation
int displayed_frame = 0;        // which frame of the animation is currently shown
int animmode = 0;				// are we currently running an animation

void update_display() {
	
	int width;
	int height;
	int frames;
	char buf[128];
	GtkLabel *label;
	GtkImage *image;
	GtkEntry *entry;
	GdkPixbuf *pixbuf;
	guchar *pb_pixels;
	char *anim_pixels;

	frames = get_num_frames_in_animation(animation);
	if (displayed_frame >= frames) displayed_frame = 0;
	if (displayed_frame < 0) displayed_frame = frames - 1;
	
	// update the label showing which frame is shown
	snprintf(buf, 128, "Frame %i of %i", displayed_frame+1, frames);
	label = GTK_LABEL(gtk_builder_get_object (builder, "framenum_label"));
	gtk_label_set_text(label, buf);

	// update the actual image
	width = get_width(animation);
	height = get_height(animation);
	image = GTK_IMAGE(gtk_builder_get_object(builder, "image1"));
	gtk_image_clear(image);
	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, width, height);
	pb_pixels = gdk_pixbuf_get_pixels(pixbuf);
	anim_pixels = get_pixels(animation, displayed_frame);
	// TODO handle rowstride != 3*width
	assert(gdk_pixbuf_get_rowstride(pixbuf) == 3*width);
	memcpy(pb_pixels, anim_pixels, width*height*3);
	gtk_image_set_from_pixbuf(image, pixbuf);
	g_object_unref(pixbuf);

	// update the duration entry
	entry = GTK_ENTRY(gtk_builder_get_object (builder, "frame_duration_entry"));
	snprintf(buf, 128, "%i", get_time_for_frame(animation, displayed_frame));
	gtk_entry_set_text(entry, buf);
}

void activate_gui_on_load() {

	GtkEntry *entry;
	GtkButton *button;
	GtkMenuItem *menuitem;

	button = GTK_BUTTON(gtk_builder_get_object(builder, "play_animation_button"));
	gtk_button_set_label(button, "Play Animation");
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "next_frame_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "prev_frame_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "set_duration_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	entry = GTK_ENTRY(gtk_builder_get_object(builder, "frame_duration_entry"));
	gtk_widget_set_sensitive(GTK_WIDGET(entry), TRUE);
	menuitem = GTK_MENU_ITEM(gtk_builder_get_object(builder, "save_menuitem"));
	gtk_widget_set_sensitive(GTK_WIDGET(menuitem), TRUE);
	menuitem = GTK_MENU_ITEM(gtk_builder_get_object(builder, "save_as_menuitem"));
	gtk_widget_set_sensitive(GTK_WIDGET(menuitem), TRUE);
	menuitem = GTK_MENU_ITEM(gtk_builder_get_object(builder, "view_menuitem"));
	gtk_widget_set_sensitive(GTK_WIDGET(menuitem), TRUE);
	menuitem = GTK_MENU_ITEM(gtk_builder_get_object(builder, "edit_menuitem"));
	gtk_widget_set_sensitive(GTK_WIDGET(menuitem), TRUE);
	menuitem = GTK_MENU_ITEM(gtk_builder_get_object(builder, "export_menuitem"));
	gtk_widget_set_sensitive(GTK_WIDGET(menuitem), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "replace_frame_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "dump_frame_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "replace_all_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "dump_all_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "replace_frame_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "dump_frame_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "replace_all_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	button = GTK_BUTTON(gtk_builder_get_object(builder, "dump_all_button"));
	gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);

}

gboolean timer_callback(gpointer data) {

	displayed_frame++;
	update_display();
	if (animmode == 1) g_timeout_add(get_time_for_frame(animation, displayed_frame), timer_callback, NULL);

	return FALSE;

}

void replace_frame_with_file(int numframe, char* filename) {

	int x, y;
	guchar *pixels, *p;
	GError *err = NULL;
	GdkPixbuf * pixbuf;
	GtkWidget *err_dialog;
	int rowstride, n_channels;

	pixbuf = gdk_pixbuf_new_from_file_at_scale(filename, get_width(animation), get_height(animation), FALSE, &err);
	
	if (err != NULL) {
		err_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
		                                    GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, 
											"Unable to read image %s: %s\n", filename, err->message);
		gtk_dialog_run(GTK_DIALOG(err_dialog));
		gtk_widget_destroy(err_dialog);
		g_error_free(err);
		return;
	}

	g_assert(gdk_pixbuf_get_colorspace (pixbuf) == GDK_COLORSPACE_RGB);
	g_assert(gdk_pixbuf_get_bits_per_sample (pixbuf) == 8);
			
	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
	pixels = gdk_pixbuf_get_pixels (pixbuf);		
	
	// it might look inefficient to set every single pixel, however,
	// since there could be an alpha channel this is neccessary	
	for (x=0; x<get_width(animation); x++) {
		for (y=0; y<get_height(animation); y++) {
			p = pixels + y * rowstride + x * n_channels;
			set_pixel(animation, numframe, x, y, p[0], p[1], p[2]);
		}
	}

}

void save_frame_to_file(int numframe, char* filename, char* type) {

	guchar *pb_pixels;
	char *anim_pixels;
	int width, height;
	GError *err = NULL;
	GdkPixbuf * pixbuf;
	GtkWidget *err_dialog;
	
	width = get_width(animation);
	height = get_height(animation);
	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, width, height);
	pb_pixels = gdk_pixbuf_get_pixels(pixbuf);
	anim_pixels = get_pixels(animation, numframe);
	// TODO handle rowstride != 3*width
	assert(gdk_pixbuf_get_rowstride(pixbuf) == 3*width);
	memcpy(pb_pixels, anim_pixels, width*height*3);
	gdk_pixbuf_save(pixbuf, filename, type, &err, NULL);
	g_object_unref(pixbuf);
	
	if (err != NULL) {
		err_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
		                                    GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, 
											"Unable to save image %s: %s\n", filename, err->message);
		gtk_dialog_run(GTK_DIALOG(err_dialog));
		gtk_widget_destroy(err_dialog);
		g_error_free(err);
		return;
	}

}

void on_play_animation_button_clicked(GtkObject *object, gpointer user_data) {

	GtkButton *button;
	GtkMenuItem *menuitem;
	
	button = GTK_BUTTON(gtk_builder_get_object(builder, "play_animation_button"));
	menuitem = GTK_MENU_ITEM(gtk_builder_get_object(builder, "play_animation_menuitem"));
	
	if (animmode == 0) {
		animmode = 1;
		gtk_button_set_label(button, "Stop Animation");
		gtk_menu_item_set_label(menuitem, "Stop Animation");
		g_timeout_add(get_time_for_frame(animation, displayed_frame), timer_callback, NULL);
	}
	else {
		animmode = 0;
		gtk_button_set_label(button, "Play Animation");
		gtk_menu_item_set_label(menuitem, "Play Animation");
	}


}

void on_next_frame_button_clicked(GtkObject *object, gpointer user_data) {
	
	displayed_frame++;
	update_display();

}

void on_prev_frame_button_clicked(GtkObject *object, gpointer user_data) {
	
	displayed_frame--;
	update_display();

}

void on_set_duration_button_clicked(GtkObject *object, gpointer user_data) {
	
	int duration;
	GtkEntry *entry;
	
	entry = GTK_ENTRY(gtk_builder_get_object (builder, "frame_duration_entry"));
	duration = atoi(gtk_entry_get_text(entry));
	set_time_for_frame(animation, displayed_frame, duration);

}

void on_about_menuitem_activate(GtkObject *object, gpointer user_data) {

	FILE *file;
	char *licence;
	size_t filesize;
	GtkWidget *dialog;

	dialog = gtk_about_dialog_new();
	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(dialog), "CowonEdit");
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), "0.1");
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog), "(c) 2010 - Timo Schneider  <timos@hrz.tu-chemnitz.de>");
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(dialog), "A program for editing the startup animation on Cowon iAudio7 players");
	file = fopen(LICENCEPATH, "r");
	fseek(file, 0L, SEEK_END);
	filesize = ftell(file);
	fseek(file, 0L, SEEK_SET);
	licence = (char*) malloc(filesize+1);
	fread(licence, 1, filesize, file);
	licence[filesize] = '\0';
	fclose(file);
	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(dialog), licence);
	gtk_about_dialog_set_url_hook(NULL, NULL, NULL);
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog), "http://perlplexity.org/Hobbies/Coding/CowonEdit");

	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
	free(licence);

}

void on_save_as_menuitem_activate(GtkObject *object, gpointer user_data) {
	
	GtkWidget *dialog;
	GtkFileFilter *filefilter;

	dialog = gtk_file_chooser_dialog_new ("Save File", 
	                                      NULL,
	                                      GTK_FILE_CHOOSER_ACTION_SAVE,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	filefilter = gtk_file_filter_new();
	gtk_file_filter_set_name(filefilter, "*.ilb Cowon Logo File");
	gtk_file_filter_add_pattern(filefilter, "*.ilb");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filefilter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		write_animation_to_file(animation, filename);
		free(last_filename);
		last_filename = (char*) malloc(strlen(filename)*sizeof(char));
		strcpy(last_filename, filename);
		g_free(filename);
	}
	gtk_widget_destroy (dialog);

}

void on_save_menuitem_activate(GtkObject *object, gpointer user_data) {
	
	if (last_filename != NULL) {
		write_animation_to_file(animation, last_filename);
	}
	else {
		on_save_as_menuitem_activate(NULL, NULL);
	}

}

void on_new_menuitem_activate(GtkObject *object, gpointer user_data) {

	GtkVBox *box;
	int numframes;
	GtkLabel *label;
	GtkEntry *entry;
	GtkWidget *dialog;

	dialog = gtk_dialog_new_with_buttons ("Create New Animation", NULL, GTK_DIALOG_MODAL,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
										  GTK_STOCK_NEW, GTK_RESPONSE_ACCEPT, NULL);
	gtk_container_set_border_width( GTK_CONTAINER( dialog ), 10 );
	label = GTK_LABEL(gtk_label_new("How many frames do you want in the new animation"));
	entry = GTK_ENTRY(gtk_entry_new());
	box = GTK_VBOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog)));
	gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(label), TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(entry), TRUE, TRUE, 0);
	
	gtk_widget_show_all(dialog);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		numframes = atoi(gtk_entry_get_text(entry));
		if (numframes < 1) numframes = 1;
		displayed_frame = 0;
		free(last_filename);
		last_filename = NULL;
		animmode = 0;
		destroy_animation(animation);
		animation = create_new_animation(numframes, 60);
		activate_gui_on_load();
		update_display();
	}
	gtk_widget_destroy (dialog);
}

void on_dump_frame_button_clicked(GtkObject *object, gpointer user_data) {

	GtkWidget *dialog;
	GtkFileFilter *filefilter;
	int numframe = displayed_frame;

	dialog = gtk_file_chooser_dialog_new ("Export File", 
	                                      NULL,
	                                      GTK_FILE_CHOOSER_ACTION_SAVE,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	filefilter = gtk_file_filter_new();
	gtk_file_filter_set_name(filefilter, "Images files");
	gtk_file_filter_add_pattern(filefilter, "*.png");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filefilter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (dialog));
		save_frame_to_file(numframe, filename, "png");
	}

	gtk_widget_destroy (dialog);
}

void on_replace_frame_button_clicked(GtkObject *object, gpointer user_data) {

	GtkWidget *dialog;
	GtkFileFilter *filefilter;
	int numframe = displayed_frame;



	dialog = gtk_file_chooser_dialog_new ("Open File", 
	                                      NULL,
	                                      GTK_FILE_CHOOSER_ACTION_OPEN,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	filefilter = gtk_file_filter_new();
	gtk_file_filter_set_name(filefilter, "Images files");
	gtk_file_filter_add_pattern(filefilter, "*.png");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filefilter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		replace_frame_with_file(numframe, filename);		
		update_display();
	}

	gtk_widget_destroy (dialog);
}

void on_open_menuitem_activate(GtkObject *object, gpointer user_data) {

	GtkEntry *entry;
	GtkButton *button;
	GtkWidget *dialog;
	GtkMenuItem *menuitem;
	GtkFileFilter *filefilter;
	
	dialog = gtk_file_chooser_dialog_new ("Open File", 
	                                      NULL,
	                                      GTK_FILE_CHOOSER_ACTION_OPEN,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	filefilter = gtk_file_filter_new();
	gtk_file_filter_set_name(filefilter, "*.ilb Cowon Logo File");
	gtk_file_filter_add_pattern(filefilter, "*.ilb");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filefilter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		destroy_animation(animation);
		animation = read_animation_from_file(filename);
		free(last_filename);
		last_filename = (char*) malloc(strlen(filename)*sizeof(char));
		strcpy(last_filename, filename);
		g_free(filename);
		displayed_frame = 0;
		animmode = 0;
		activate_gui_on_load();
		update_display();
	}
	gtk_widget_destroy (dialog);

}

void on_replace_all_button_clicked(GtkObject *object, gpointer user_data) {

	GtkVBox *box;
	char *dirname;
	glob_t globbuf;
	GtkLabel *label;
	GtkEntry *entry;
	GtkWidget *dialog;
	GtkWidget *dialog2;

	dialog = gtk_file_chooser_dialog_new ("Select Folder", 
	                                      NULL,
	                                      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		dirname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (dialog));
		
		// ask the user for the name pattern
		dialog2 = gtk_dialog_new_with_buttons ("Set Filename Pattern", NULL, GTK_DIALOG_MODAL,
		                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
		                                       GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
		gtk_container_set_border_width( GTK_CONTAINER( dialog2 ), 10 );
		label = GTK_LABEL(gtk_label_new(NULL));
		gtk_label_set_markup(label, "<b>Set a name pattern for the frames to import</b>\n\n"
		                            "In the pattern, use %1 for the frame number and %2\n"
		                            "for the frame duration.\n\n"
		                            "<b>Example: mylogo_%1_%2.png</b> the first frame will\n"
									"be loaded from mylogo_1_*.png (i.e. mylogo_1_60.png if\n"
									"that file exists) and have a duration of 60 ms in the\n"
									"animation).\n\n"
									"Note: If a frame exists in multiple duration versions we\n"
									"pick the first one returned by glob(3). If no file can\n"
									"be found for a frame, this frame is left unchanged.\n");
		entry = GTK_ENTRY(gtk_entry_new());
		box = GTK_VBOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog2)));
		gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(label), TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(entry), TRUE, TRUE, 0);
	
		gtk_widget_show_all(dialog2);
	
		if (gtk_dialog_run (GTK_DIALOG(dialog2)) == GTK_RESPONSE_ACCEPT) {
			
			int frame;
			int bufsize;
			int duration;
			char buffer[256];
			char* pattern, *p, *b;
			
			for (frame=0; frame < get_num_frames_in_animation(animation); frame++) {
				// construct filename from pattern
				pattern = (char*) gtk_entry_get_text(entry);
				b = buffer;
				p = pattern;
				duration = get_time_for_frame(animation, frame);
				while ((*p != '\0') && (b < buffer + 255)) {
					if ((*p == '%') && (*(p+1) == '1')) {
						b += sprintf(b, "%i", frame+1);
						p += 2;
					}
					else if ((*p == '%') && (*(p+1) == '2')) {
						b += sprintf(b, "*", duration);
						p += 2;
					}				
					else {
						*b = *p;
						p++;
						b++;
					}
				}
				*b = '\0';
				
				// concatenate dirname with filename
				b = (char*) malloc(strlen(dirname)+strlen(buffer)+2);
				p = b;
				strcpy(b, dirname);
				b += strlen(dirname);
				strcpy(b, "/");
				b++;
				strcpy(b, buffer);
				
				if (glob(p, GLOB_NOESCAPE, NULL, &globbuf) != GLOB_NOMATCH) {
					
					int beg, end;

					replace_frame_with_file(frame, globbuf.gl_pathv[0]);
					// get duration by comparing glob with match and set it
					for (beg=0; beg<strlen(p); beg++) {
						if (*(p+beg) != *(globbuf.gl_pathv[0]+beg)) break;
					}
					for (end=0; end<strlen(p); end++) {
						if (*(p+strlen(p)-end) != *(globbuf.gl_pathv[0]+strlen(globbuf.gl_pathv[0])-end)) break;
					}
					*(globbuf.gl_pathv[0]+strlen(globbuf.gl_pathv[0])-end+1) = '\0';
					duration = atoi(globbuf.gl_pathv[0]+beg);
					set_time_for_frame(animation, frame, duration);
				}
				globfree(&globbuf);
				free(p);
			}
			
		}
		
		gtk_widget_destroy(dialog2);

	}

	gtk_widget_destroy(dialog);
	update_display();
}

void on_dump_all_button_clicked(GtkObject *object, gpointer user_data) {

	GtkVBox *box;
	char *dirname;
	GtkLabel *label;
	GtkEntry *entry;
	GtkWidget *dialog;
	GtkWidget *dialog2;

	dialog = gtk_file_chooser_dialog_new ("Select Folder", 
	                                      NULL,
	                                      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
	                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		dirname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (dialog));
		
		// ask the user for the name pattern
		dialog2 = gtk_dialog_new_with_buttons ("Set Filename Pattern", NULL, GTK_DIALOG_MODAL,
		                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
		                                       GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
		gtk_container_set_border_width( GTK_CONTAINER( dialog2 ), 10 );
		label = GTK_LABEL(gtk_label_new(NULL));
		gtk_label_set_markup(label, "<b>Set a name pattern for the exported frames</b>\n\n"
		                            "In the pattern, use %1 for the frame number and %2\n"
		                            "or the frame duration\n\n"
		                            "<b>Example: mylogo_%1_%2.png</b> will save the first\n"
		                            "frame as mylogo_1_60.png (assumed that frame was\n"
		                            "60 ms long in the animation)\n\n"
									"Note: You don't need to use %2, but you should really\n"
									"use %1 otherwise the last frame will overwrite all\n"
									"previous ones that have the same name (because of the\n"
									"same duration or an omission of %2).\n");
		entry = GTK_ENTRY(gtk_entry_new());
		box = GTK_VBOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog2)));
		gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(label), TRUE, TRUE, 0);
		gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(entry), TRUE, TRUE, 0);
	
		gtk_widget_show_all(dialog2);
	
		if (gtk_dialog_run (GTK_DIALOG(dialog2)) == GTK_RESPONSE_ACCEPT) {
			
			int frame;
			int bufsize;
			int duration;
			char buffer[256];
			char* pattern, *p, *b;
			
			for (frame=0; frame < get_num_frames_in_animation(animation); frame++) {
				// construct filename from pattern
				pattern = (char*) gtk_entry_get_text(entry);
				b = buffer;
				p = pattern;
				duration = get_time_for_frame(animation, frame);
				while ((*p != '\0') && (b < buffer + 255)) {
					if ((*p == '%') && (*(p+1) == '1')) {
						b += sprintf(b, "%i", frame+1);
						p += 2;
					}
					else if ((*p == '%') && (*(p+1) == '2')) {
						b += sprintf(b, "%i", duration);
						p += 2;
					}				
					else {
						*b = *p;
						p++;
						b++;
					}
				}
				*b = '\0';
				
				// concatenate dirname with filename
				b = (char*) malloc(strlen(dirname)+strlen(buffer)+2);
				p = b;
				strcpy(b, dirname);
				b += strlen(dirname);
				strcpy(b, "/");
				b++;
				strcpy(b, buffer);
				save_frame_to_file(frame, p, "png");
				free(p);
			}
			
		}
		
		gtk_widget_destroy(dialog2);

	}

	gtk_widget_destroy(dialog);

}

int main(int argc, char **argv) {

	GtkWidget *window;
	
	gtk_init (&argc, &argv);
	builder = gtk_builder_new();
	gtk_builder_add_from_file(builder, "cowonedit.glade", NULL);
	window = GTK_WIDGET(gtk_builder_get_object(builder, "main_window"));
	gtk_builder_connect_signals(builder, NULL);
	gtk_widget_show (window);
	gtk_main ();
	
	return 0;
	
}

