#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <inttypes.h>

#include "libcowonilb.h"

CowonAnim* create_new_animation(unsigned int numframes, unsigned int default_time_per_frame) {

	int i;
	
	CowonAnim *a = (CowonAnim *) malloc(sizeof(struct ilb) + numframes*2 + numframes*160*128*3);
	if (a == NULL) {
		fprintf(stderr, "Couldn't allocate memory\n");
		return NULL;
	}
	strcpy(a->header, "iAUDIO 160x128C\0");
	a->unknown = 1;
	a->numframes = numframes;
	char* p = ((char*) a) + sizeof(CowonAnim);
	for (i=0; i<numframes; i++) *((uint16_t*) p+i) = default_time_per_frame;
	
	return a;
}

void destroy_animation(CowonAnim* a) {
	
	free(a);

}

int get_num_frames_in_animation(CowonAnim* a) {
	
	if (a != NULL) return a->numframes;
	else {
		fprintf(stderr, "not a valid cowonanimation structure, got NULL pointer\n");
		return -1;
	}

}

unsigned int get_time_for_frame(CowonAnim* a, unsigned int frame) {
	
	if (a != NULL) {
		if (frame < a->numframes) {
			uint16_t *t = (uint16_t*) (((char*) a) + sizeof(struct ilb) + frame*2);
			return *t;
		}
		else {
			fprintf(stderr, "request for frame %i in an animation with %i frames\n", frame, a->numframes);
			return -1;
		}
	}
	else {
		fprintf(stderr, "not a valid cowonanimation structure, got NULL pointer\n");
		return -1;
	}

}

unsigned int set_time_for_frame(CowonAnim* a, unsigned int frame, unsigned int time_in_ms) {
	
	if (a != NULL) {
		if (frame < a->numframes) {
			uint16_t *t = (uint16_t*) (((char*) a) + sizeof(struct ilb) + frame*2);
			*t = time_in_ms;
		}
		else {
			fprintf(stderr, "request for frame %i in an animation with %i frames\n", frame, a->numframes);
			return -1;
		}
	}
	else {
		fprintf(stderr, "not a valid cowonanimation structure, got NULL pointer\n");
		return -1;
	}

}

void write_animation_to_file(CowonAnim *a, char* filename) {
	
	FILE *file;

	if (a == NULL) {
		fprintf(stderr, "not a valid cowonanimation structure, got NULL pointer\n");
		return;
	}
	  
	file = fopen(filename, "w");
	if (file == NULL) {
		perror("Error opening file for writing");
		return;
	}
	int ret = fwrite(a, 1, sizeof(struct ilb) + a->numframes*2 + a->numframes*get_width(a)*get_height(a)*3, file);
	if (ret != sizeof(struct ilb) + a->numframes*2 + a->numframes*get_width(a)*get_height(a)*3) {
		fprintf(stderr, "Couldn't write image data\n");
	}
	fclose(file);

}

void set_pixel(CowonAnim *a, unsigned int frame, unsigned int x, unsigned int y, unsigned char red, unsigned char green, unsigned char blue) {
	
	if ((x>=get_width(a)) || (y>=get_height(a)) || (a==NULL) || (frame >= a->numframes)) {
		fprintf(stderr, "Inavlid arguments to set_pixel\n");
		if (x>=get_width(a)) fprintf(stderr, "x value must be smaller than %i, is %i\n",get_width(a), x);
		if (y>=get_height(a)) fprintf(stderr, "y value must be smaller than %i, is %i\n",get_height(a), y);
		if (a==NULL) fprintf(stderr, "not a valid cowonanimation structure, got NULL pointer\n");
		if (frame >= a->numframes) fprintf(stderr, "frame must be smaller than %i for that animation\n", a->numframes);
		
		return;
	}

	char *p = ((char*) a) + sizeof(struct ilb) + a->numframes*2 + frame*get_width(a)*get_height(a)*3 + y*get_width(a)*3 + x*3;
	p[0] = red;
	p[1] = green;
	p[2] = blue;
 
}

char* get_pixels(CowonAnim *a, unsigned int frame) {
	
	if ((a==NULL) || (frame >= a->numframes)) {
		fprintf(stderr, "Inavlid arguments to set_pixel\n");
		if (a==NULL) fprintf(stderr, "not a valid cowonanimation structure, got NULL pointer\n");
		if (frame >= a->numframes) fprintf(stderr, "frame must be smaller than %i for that animation\n", a->numframes);
		
		return;
	}

	char *p = ((char*) a) + sizeof(struct ilb) + a->numframes*2 + frame*get_width(a)*get_height(a)*3;

	return p; 
}

int get_width(CowonAnim* a) {

	char *p;

	p = &(a->header[7]);
	
	return atoi(p);

}

int get_height(CowonAnim* a) {
	
	char *p;

	p = &(a->header[7]);
	while (isdigit(*p)) p++;
	p++;
	
	return atoi(p);

}

CowonAnim* read_animation_from_file(char* filename) {
	
	int ret;
	FILE *file;
	CowonAnim* a;
	char header[6];
	size_t filesize;

	file = fopen(filename, "r");
		if (file == NULL) {
		perror("Error opening file for reading");
		return;
	}
	ret = fread(header, 1, 6, file);
	if (ret != 6) {
		fprintf(stderr, "Couldn't read image data\n");
		return NULL;
	}
	else if (strncmp(header, "iAUDIO", 6) != 0) {
		fprintf(stderr, "Data doesn't look like a cowon ilb animation\n");
		return;
	}

	fseek(file, 0L, SEEK_END);
	filesize = ftell(file);
	fseek(file, 0L, SEEK_SET);
	a = (CowonAnim*) malloc(filesize);
	ret = fread(a, 1, filesize, file);
	if (ret != filesize) {
		fprintf(stderr, "Couldn't read image data\n");
		return NULL;
	}
	fclose(file);

	return a;

}
